import _ from 'lodash';
import configureStore from 'redux-mock-store';
import {hoc} from '../';
import {mount} from 'enzyme';
import thunk from 'redux-thunk';

describe('List HOC', () => {
	let
		store;

	beforeEach(() => { store = configureStore([thunk])({}); });

	it('syncs remote data with redux store being applied with [fetchOnMount === true]', () => {
		mount(hoc.list({
			baseURL     : '/',
			entity      : 'contacts',
			fetchOnMount: true,
			key         : 'contacts',
		})(() => null)({store}));

		expect(store.getActions().pop()).toEqual({
			request: {params: {}, pathvars: {page: 1}},
			syncing: true,
			type   : '@@redux-api@contacts',
		});
	});

	it('doesn\'t sync remote data with redux store being applied with [fetchOnMount === false]', () => {
		mount(hoc.list({
			baseURL     : '/',
			entity      : 'contacts',
			fetchOnMount: false,
			key         : 'contacts',
		})(() => null)({store}));

		expect(store.getActions().pop()).toEqual(undefined);
	});

	it('syncs remote data on [queryParams] change', () => {
		let
			instance = mount(hoc.list({
				baseURL    : '/',
				entity     : 'contacts',
				key        : 'contacts',
				queryParams: props => ({testParam: props.testProp}),
			})(() => null)({store, testProp: 'a'}));

		instance.setProps({store, testProp: 'b'});

		expect(store.getActions().pop()).toEqual({
			request: {params: {}, pathvars: {page: 1, testParam: 'b'}},
			syncing: true,
			type   : '@@redux-api@contacts',
		});
	});

	it('syncs remote data on [urlParams] change', () => {
		let
			instance = mount(hoc.list({
				baseURL  : '/',
				entity   : 'contact',
				key      : 'contact',
				urlParams: props => ({id: props.id, type: props.type}),
			})(() => null)({id: 'initialid', store, type: 'initialtype'}));

		instance.setProps({id: 'newid', store, type: 'newtype'});

		expect(store.getActions().pop()).toEqual({
			request: {params: {}, pathvars: {id: 'newid', page: 1, type: 'newtype'}},
			syncing: true,
			type   : '@@redux-api@contact',
		});
	});

	it('syncs remote data on [page] change', () => {
		const
			requestedPage = 35;

		store = configureStore([thunk])(() => ({
			some: {path: {page: (_.first(store.getActions()) || {}).page || 33}},
		}));

		// Page value changes in store
		mount(hoc.list({
			baseURL: '/',
			entity : 'contacts',
			key    : 'contacts',
			page   : 'some.path.page',
		})(() => null)({store}));

		store.dispatch({page: requestedPage, type: 'TRIGGER_PAGE_CHANGE'});

		expect(store.getActions().pop()).toEqual({
			request: {params: {}, pathvars: {page: requestedPage}},
			syncing: true,
			type   : '@@redux-api@contacts',
		});
	});
});
